/* Test of polynomials division
 * Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2011/January/10
 */

#include "polynom.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


typedef struct
{
    int  rc;   /* program return code */
    polynom_t  *p1;
    polynom_t  *p2;
    polynom_t  *p3;
    polynom_t  *p13;
    polynom_t  *p23;
} test_t;


static void usage(char *p0)
{
    fprintf(stderr,
            "Usage: %s -p a[m]...a[0] -p b[n]...b[0] -p c[n]...c[0]\n", p0);
}


static polynom_t  *args2poly(test_t *t, char **argv, int ai, int na)
{
    int        k;
    int        degree =   na - 1;
    polynom_t  *p = polynom_create(degree);

    /* Assume good numbers */
    for (k = degree;  k >= 0;  k--, ai++)
    {
        p->coefs[k] = strtod(argv[ai], NULL);
    }

    return p;
}


static void test_init(test_t  *t, int argc, char **argv)
{
    int  dash_p[3], n_dash_p, ai;

    t->rc = 0;
    t->p1 = NULL;
    t->p2 = NULL;
    t->p3 = NULL;
    t->p13 = NULL;
    t->p23 = NULL;

    /* Fing positions of '-p' in argv[] */
    for (ai = 1, n_dash_p = 0; t->rc == 0 && ai < argc && n_dash_p < 3; ai++)
    {
        if (strcmp(argv[ai], "-p") == 0)
        {
            if ((n_dash_p > 0) && ((ai - dash_p[n_dash_p - 1]) == 1))
            {
                t->rc = 1; /* empty polynomial, two consecutive "-p"s. */
            }
            dash_p[n_dash_p++] = ai;
        }
    }

    if (t->rc == 0 && n_dash_p == 3)
    {
        t->p1 = args2poly(t, argv, dash_p[0] + 1, dash_p[1] - (dash_p[0] + 1));
        t->p2 = args2poly(t, argv, dash_p[1] + 1, dash_p[2] - (dash_p[1] + 1));
        t->p3 = args2poly(t, argv, dash_p[2] + 1, argc      - (dash_p[2] + 1));
        t->p13 = polynom_create(0);
        t->p23 = polynom_create(0);
        polynom_multiply(t->p13, t->p1, t->p3);
        polynom_multiply(t->p23, t->p2, t->p3);
    }
    else
    {
        fprintf(stderr, "Missing polynomials coefficients\n");
        t->rc = 1;
    }
}


static void  test_finit(test_t  *t)
{
    if (t->p1) { polynom_destroy(t->p1); }
    if (t->p2) { polynom_destroy(t->p2); }
    if (t->p3) { polynom_destroy(t->p3); }
    if (t->p13) { polynom_destroy(t->p13); }
    if (t->p23) { polynom_destroy(t->p23); }
}


static void  wrap_poly_print(char *before, polynom_t *p, char *after)
{
    printf("%s", before);
    polynom_print(stdout, p);
    printf("%s", after);
}


void test_run(test_t *t)
{
    polynom_t *gcd = polynom_create(0);

    wrap_poly_print("p1(x) = ", t->p1, "\n");
    wrap_poly_print("p2(x) = ", t->p2, "\n");
    wrap_poly_print("p3(x) = ", t->p3, "\n\n");
    wrap_poly_print("p13(x) = ", t->p13, "\n");
    wrap_poly_print("p23(x) = ", t->p23, "\n");

    polynom_gcd(gcd, t->p13, t->p23);
    wrap_poly_print("(gcd(p13,p23))(x) = ", gcd, "\n");
    
    if (!polynom_is_divider(t->p3, gcd))
    {
        t->rc = 1;
        fprintf(stderr, "Failed: p3 | gcd\n");
    }
    else if (!polynom_is_divider(gcd, t->p13))
    {
        t->rc = 1;
        fprintf(stderr, "Failed: gcd | p13\n");
    }
    else if (!polynom_is_divider(gcd, t->p23))
    {
        t->rc = 1;
        fprintf(stderr, "Failed: gcd | p23\n");
    }
    polynom_destroy(gcd);
}


int main(int argc, char **argv)
{
    test_t  t;

    test_init(&t, argc, argv);
    if (t.rc != 0)
    {
        usage(argv[0]);
    }
    else
    {
        test_run(&t);
    }
    test_finit(&t);

    return t.rc;
}
