/* Test of polynom package 
 * Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2010/December/20
 */

#include "polynom.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


typedef struct
{
    int  rc;   /* program return code */
    polynom_t  *p;
    polynom_t  *q;
} test_t;


static void usage(char *p0)
{
    fprintf(stderr, 
            "Usage: %s -p a[m] a[m-1] ... a[0] -p b[n] b[n-1] ... b[0]\n", p0);
}


static polynom_t  *args2poly(test_t *t, char **argv, int ai, int na)
{
    int        k;
    int        degree =   na - 1;
    polynom_t  *p = polynom_create(degree);

    /* Assume good numbers */
    for (k = degree;  k >= 0;  k--, ai++)
    {
        p->coefs[k] = strtod(argv[ai], NULL);
    }

    return p;
}


static void test_init(test_t  *t, int argc, char **argv)
{
    int  ai, p_1st = 0, p_2nd = 0;
    int  p_ai_begin, p_na;
    int  q_ai_begin, q_na;

    t->rc = 0;
    t->p = NULL;
    t->q = NULL;

    /* find 1st '-p' */
    for (ai = 1;  ai < argc && (strcmp(argv[ai], "-p") != 0);  ai++);
    if (ai < argc) { p_1st = ai; }
    /* find 2nd '-p' */
    for (ai++;  ai < argc && (strcmp(argv[ai], "-p") != 0);  ai++);
    if (ai < argc) { p_2nd = ai; }

    /*  To have 2 polynomials, we must have two "-p" arguments,
     *  and at least one number after them.
     */
    p_ai_begin = p_1st + 1;   p_na = p_2nd - p_ai_begin;
    q_ai_begin = p_2nd + 1;   q_na = argc  - q_ai_begin;
    if ((p_na > 0) && (q_na > 0))
    {
        t->p = args2poly(t, argv, p_1st + 1,  p_2nd - (p_1st + 1));
        t->q = args2poly(t, argv, p_2nd + 1,  argc  - (p_2nd + 1));
    }
    else
    {
        fprintf(stderr, "Missing polynomials coefficients\n");
        t->rc = 1;
    }
}


static void  test_finit(test_t  *t)
{
    if (t->p) { polynom_destroy(t->p); }
    if (t->q) { polynom_destroy(t->q); }
}


static void  wrap_poly_print(char *before, polynom_t *p, char *after)
{
    printf("%s", before);  
    polynom_print(stdout, p);
    printf("%s", after);  
}


void test_run(test_t *t)
{
    polynom_t *result;

    wrap_poly_print("p(x) = ", t->p, "\n");
    wrap_poly_print("q(x) = ", t->q, "\n");
    
    result = polynom_create_constant(0);

    polynom_add(result, t->p, t->q);
    wrap_poly_print("(p + q)(x) = ", result, "\n");

    polynom_subtract(result, t->p, t->q);
    wrap_poly_print("(p - q)(x) = ", result, "\n");

    polynom_multiply(result, t->p, t->q);
    wrap_poly_print("(pq)(x) = ", result, "\n");

    polynom_destroy(result);
}


int main(int argc, char **argv)
{
    test_t  t;

    test_init(&t, argc, argv);
    if (t.rc != 0) 
    { 
        usage(argv[0]); 
    }
    else
    {
        test_run(&t);
    }
    test_finit(&t);

    return t.rc;
}
